C
C     *******************************************************
C     *                                                     *
C     *                         ATrans3                     *
C     *                                                     *
C     *******************************************************
C
C     DESCRIPTION
C     ===========
C     ANALYTICAL SOLUTION FOR 3D ADVECTIVE-DISPERSIVE TRANSPORT
C     FROM A PATCH SOURCE ALONG THE X-BOUNDARY.
C
C            0  <= X  <  INFINITY
C     -INFINITY <  Y  <  INFINITY
C            0  <= Z  <= B
C            T  >  0
C
C     --> MODEL 3: THE PATCH HAS AN ARBITRARY TIME-VARYING
C                  CONCENTRATION SPECIFIED BY A SET OF POINTS
C
C     ANALYTICAL SOLUTION : C.J. NEVILLE 94/04
C     FORTRAN77 CODE      : C.J. NEVILLE 97/11
C
C     DEFINITION OF INPUT PARAMETERS
C     ==============================
C     V       : AVG. LINEAR GROUNDWATER VELOCITY IN X-DIRECTION
C     ALX     : LONGITUDINAL DISPERSIVITY
C     ALY     : HORIZONTAL TRANSVERSE DISPERSIVITY
C     ALZ     : VERTICAL TRANSVERSE DISPERSIVITY
C     DSTAR   : EFFECTIVE DIFFUSION COEFFICIENT
C     THICK   : AQUIFER THICKNESS
C     CLAMDA  : FIRST-ORDER DECAY COEFFICIENT
C     R       : RETARDATION FACTOR
C     SWIDTH  : WIDTH OF SOURCE (2Yo)
C     Z1      : ELEVATION OF BOTTOM OF SOURCE
C     Z2      : ELEVATION OF TOP OF SOURCE
c
c     atrans3: input for time-varying patch concentration
c     ---------------------------------------------------
C     NP      : NUMBER OF POINTS DEFINING INFLOW CONCENTRATION HISTORY
C     TSI,CSI : INFLOW CONCENTRATION HISTORY POINT (TIME,CONCENTRATION)
c
c     solution parameters
c     -------------------
C     NGAUS   : NUMBER OF GAUSS POINTS FOR INTEGRATION
C     NFOUR   : NUMBER OF TERMS IN INFINITE SERIES
C
C     DECLARATION OF VARIABLES
C     ========================
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
      CHARACTER*80 JOBID,TITLE,RECORD*19,TEXT*16
      REAL T2,RCONC
C
c     set variable dimensions in atrans.inc
c     -------------------------------------
      INCLUDE 'ATRANS.INC'
c
C     MAXNP   : MAXIMUM NUMBER OF POINTS DESCRIBING INFLOW
C               CONCENTRATION HISTORY
C     MAXobs  : MAXIMUM NUMBER OF observation points
C     MAXT    : MAXIMUM NUMBER OF TIMES
C     MAXX    : MAXIMUM NUMBER OF X-POINTS
C     MAXY    : MAXIMUM NUMBER OF Y-POINTS
C     MAXZ    : MAXIMUM NUMBER OF Z-POINTS
C     MAXGAUS : MAXIMUM NUMBER OF TERMS IN GAUSS-LEGENDRE QUADRATURE
C               (UPPER LIMIT IS 256)
C
      COMMON    TSI(MAXnp),CSI(MAXnp),TSTART(MAXnp),DELC(MAXnp),
     &          XI(MAXobs),YI(MAXobs),ZI(MAXobs),BTC(MAXobs),
     &          Time(MAXt),XC(MAXx),YC(MAXy),ZC(MAXz),
     &          Conc(MAXxyz),RCONC(MAXxyz)
C
      COMMON /TDATA/VR,DXR,DYR,DZR,CLAMDA,
     &              Y0,THICK,Z1,Z2,IFLAGPP
      COMMON /SPARAM/NGAUS,NFOUR
c
C     Print an identifier to screen
C     =============================
      Write(*,101)
  101 Format('+','ATrans3 - Analytical Transport from a',
     & ' Patch Source. C.J. Neville. Ver. 2.01'/)
C
C     OPEN INPUT AND OUTPUT FILES
C     ===========================
      WRITE(*,*) 'INPUT JOBID FOR FILE SPECIFICATION: '
      READ(*,1000) JOBID
      WRITE(*,*)
      LENFIL = INDEX(JOBID,' ')-1
C
      OPEN(UNIT=55,FILE=JOBID(:LENFIL)//'.INP',STATUS='OLD')
      OPEN(UNIT=60,FILE=JOBID(:LENFIL)//'.LST',STATUS='UNKNOWN')
C
C     READ INPUT PARAMETERS
C     =====================
C     TITLE
C     -----
      READ(55,1000) TITLE
C
C     TRANSPORT PARAMETERS
C     --------------------
      READ(55,*) V
      READ(55,*) ALX
      READ(55,*) ALY
      READ(55,*) ALZ
      READ(55,*) DSTAR
      READ(55,*) THICK
      READ(55,*) CLAMDA
      READ(55,*) R
C
C     SOLUTION PARAMETERS
C     -------------------
      READ(55,*) NGAUS
      READ(55,*) NFOUR
C
C     inflow patch location
C     ---------------------
      READ(55,*) SWIDTH
      READ(55,*) Z1
      READ(55,*) Z2
c
c     atrans3: inflow patch concentration history
c     -------------------------------------------
      READ(55,*) NP
C
      do npi=1, np
         READ(55,*) TSI(NPI),CSI(NPI)!,NPI=1,NP)
      enddo
C
C     data for OBSERVATION POINTs
C     ---------------------------
      READ(55,*) NOBS
C
      IF(NOBS.GT.0) THEN
         DO N=1,NOBS
            READ(55,*) XI(N),YI(N),ZI(N)
         END DO
         READ(55,*) TMIN,TMAX,DELT
      END IF
C
C     data for concentration distributions
C     ------------------------------------
      READ(55,*) NTIMES
C
      IF(NTIMES.GT.0) THEN
         READ(55,*) (Time(I),I=1,NTIMES)
         READ(55,*) XMIN,XMAX,DELX
         READ(55,*) YMIN,YMAX,DELY
         READ(55,*) ZMIN,ZMAX,DELZ
      END IF
c
      close(55)
C
C     WRITE HEADING
C     =============
      WRITE(60,2000)
      WRITE(60,2100) TITLE
C
C     ECHO INPUT PARAMETERS
C     =====================
      WRITE(60,2200) V,ALX,ALY,ALZ,DSTAR
      WRITE(60,2300) THICK
      WRITE(60,2400) CLAMDA,R
      WRITE(60,2500) NGAUS,NFOUR
      WRITE(60,2301) SWIDTH,Z1,Z2
c
C     CHECK INPUT DATA
C     ================
      CALL CHK1(Z1,Z2,THICK)
      CALL CHK2(NGAUS,NFOUR)
c
c     echo inflow concentration data
c     ==============================
      write(60,2302)
      WRITE(60,2303) (TSI(I),CSI(I),I=1,NP)
      CALL CHK3(NP,TSI)
c
C     COMPUTE INFLOW CONCENTRATION HISTOGRAM
C     ======================================
      TSTART(1) = TSI(1)
      DELC(1)   = CSI(1)
      IF(NP.GT.1) THEN
         DO NPI=2,NP
            TSTART(NPI) = (TSI(NPI)+TSI(NPI-1))/2.D0
            DELC(NPI)   = CSI(NPI)-CSI(NPI-1)
         end do
      END IF
C
C     ECHO INFLOW CONCENTRATION HISTOGRAM
C     ===================================
      WRITE(60,2600)
      DO NPI=1,NP
         IF(NPI.LT.NP) THEN
            WRITE(60,2700) TSTART(NPI),TSTART(NPI+1),CSI(NPI)
         ELSE
            WRITE(60,2800) TSTART(NPI),CSI(NPI)
         END IF
      end do
c
      close(60)
C
c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C
C     set-up for breakthrough curves
C     ==============================
      IF(NOBS.GT.0) THEN
c
         OPEN(UNIT=61,FILE=JOBID(:LENFIL)//'.OBS',STATUS='UNKNOWN')
C
         CALL CHK4(NOBS,XI,ZI,thick,tmin,tmax,dELt)
c
c        calculate number of times for breakthrough curves
c        -------------------------------------------------
         NT = IDINT(((TMAX-TMIN)/DELT)+0.5d0)+1
      END IF
c
C     set-up for CONCENTRATION DISTRIBUTIONS
C     ======================================
      IF(NTIMES.GT.0) THEN
c
         OPEN(UNIT=62,FILE=JOBID(:LENFIL)//'.CNF',STATUS='UNKNOWN')
         OPEN(UNIT=63,FILE=JOBID(:LENFIL)//'.MSH',STATUS='UNKNOWN')
         OPEN(UNIT=64,FILE=JOBID(:LENFIL)//'.ASC',STATUS='UNKNOWN')
         OPEN(UNIT=65,FILE=JOBID(:LENFIL)//'.UCN',STATUS='UNKNOWN',
     1        form='unformatted')
C
         call chk5(xmin,xmax,dELx,ymin,ymax,dELy,
     1        zmin,zmax,dELz,thick)
c
C        CALCULATE NUMBER OF POINTS FOR CONCENTRATION DISTRIBUTIONS
C        ==========================================================
         NX = IDINT(((XMAX-XMIN)/DELX)+0.5d0)+1
         NY = IDINT(((YMAX-YMIN)/DELY)+0.5d0)+1
         NZ = IDINT(((ZMAX-ZMIN)/DELZ)+0.5d0)+1
C
C        CHECK times and dimensions
C        --------------------------
         CALL CHK6(Ntimes,time,nx,ny,nz)
C
C        CALCULATE LOCATIONS FOR CONCENTRATION DISTRIBUTIONS
C        ---------------------------------------------------
         DO jj=1,nx
            XC(jj) = XMIN+DELX*DBLE(jj-1)
         end do
C
         DO ii=1,ny
            YC(ii) = YMIN+DELY*DBLE(ii-1)
         end do
C
         DO Kk=1,nz
            ZC(kK) = ZMIN+DELZ*DBLE(kK-1)
         end do
C
c        write out model config file and mesh file
c        =========================================
C        common parameters
C        -----------------
         CINACT = 1.E30
         htop   = zC(NZ)
C
c        CNF file
c        --------
         WRITE(62,'(I5,1X,I5,1X,I5)') NZ,NY,NX
         WRITE(RECORD,'(1X,I6,A1,E11.5)') NX,'*',DELX
         write(62,'(A)') RECORD
         WRITE(RECORD,'(1X,I6,A1,E11.5)') NY,'*',DELY
         write(62,'(A)') RECORD
         WRITE(RECORD,'(1X,I6,A1,E11.5)') NX*NY,'*',htop
         write(62,'(A)') RECORD
         WRITE(RECORD,'(1X,I6,A1,E11.5)') NX*NY*NZ,'*',DELZ
         write(62,'(A)') RECORD
         WRITE(62,'(1X,1PE12.3)') CINACT
c
         close(62)
c
c        msh file
c        --------
         XO     = XMIN-DELX/2.0
         YO     = YMIN-DELY/2.0
         ROTATE = 0.
c
         WRITE(63,'(I5,1X,I5,1X,I5)') NZ,NY,NX
         WRITE(RECORD,'(1X,I6,A1,E11.5)') NX,'*',DELX
         WRITE(63,'(I5,1X,I5,1X,I5)') NZ,NY,NX
         WRITE(RECORD,'(1X,I6,A1,E11.5)') NY,'*',DELY
         write(63,'(A)') RECORD
         WRITE(RECORD,'(1X,I6,A1,E11.5)') NX*NY,'*',htop
         WRITE(63,'(A)') RECORD
         WRITE(RECORD,'(1X,I6,A1,E11.5)') NX*NY*NZ,'*',DELZ
         write(63,'(A)') RECORD
         WRITE(63,'(3(1X,1PE12.3))') XO,YO,ROTATE
         WRITE(63,'(1X,1PE12.3)') CINACT
c
         close(63)
C
C        set parameters for header of unformatted concentration file
C        -----------------------------------------------------------
         ntrans = 1
         kstp   = 1
         kper   = 1
         text   = 'CONCENTRATION'
      END IF
c     xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
C
C     COMPUTE DISPERSION COEFFICIENTS
C     ===============================
      DX    = (ALX*V)+DSTAR
      DY    = (ALY*V)+DSTAR
      DZ    = (ALZ*V)+DSTAR
C
C     COMPUTE TRANSPORT CONSTANTS
C     ===========================
      VR    = V/R
      DXR   = DX/R
      DYR   = DY/R
      DZR   = DZ/R
      Y0    = SWIDTH/2.D0
C
c     determine whether the source is fully or partially penetrating
c     ==============================================================
      SMALLZ = 1.D-6
      IF((Z1.LE.SMALLZ).AND.(Z2.GE.(THICK-SMALLZ))) THEN
         IFLAGPP = 0
      ELSE
         IFLAGPP = 1
      END IF
C
C     COMPUTE BREAKTHROUGH CURVES
C     ===========================
      IF(NOBS.GT.0) THEN
c
         WRITE(*,9000)
9000     FORMAT(' CALCULATING CONCENTRATIONS FOR BTC AT TIME :',/)
C
C        LOOP OVER OUTPUT TIMES
C        ----------------------
         DO I=1,NT
            TC = TMIN+DELT*DBLE(I-1)
C
            WRITE(*,'(1h+,1PE12.3)') TC
C
C           LOOP OVER NUMBER OF OBSERVATION POINTS
C           --------------------------------------
            DO N=1,NOBS
C
C            LOOP OVER THE SOURCE CONCENTRATION HISTORY
C            ==========================================
             SUMC = 0.D0
             DO NPI=1,NP
               TSHIFT = TC-TSTART(NPI)
               IF(TSHIFT.LE.0.D0) THEN
                 DC = 0.D0
               ELSE
                 CALL MODEL1(XI(N),YI(N),ZI(N),TSHIFT,C)
                 DC = C*DELC(NPI)
               END IF
               SUMC = SUMC+DC
             end do
c
             btc(n) = sumc
C
            END DO
C
C           WRITE OUT CONCENTRATIONS AT OBSERVATION POINTS
C           ----------------------------------------------
            WRITE(61,3000) TC,(BTC(N),N=1,NOBS)
         END DO
c
         close(61)
c
      END IF
c
C     COMPUTE concentration distributions
C     ===================================
      IF(Ntimes.GT.0) THEN
C
         WRITE(*,9001)
9001     FORMAT(//' CALCULATING CONCENTRATION DISTRIBUTIONS',/)
c
C        LOOP OVER OUTPUT TIMES
C        ----------------------
         DO I=1,NTimes
C
          WRITE(*,9002) time(i)
9002      format(/,' concentration distribution at time : ',1pe12.3/)
C
C         LOOP OVER COORDINATES
C         ---------------------
          DO jj=1,nx
            do ii=1,ny
              do kk=1,nz
                WRITE(*,'(1h+,3(1PE12.3))') xc(jj),yc(ii),zc(kk)
C
C               LOOP OVER THE SOURCE CONCENTRATION HISTORY
C               ==========================================
                SUMC = 0.D0
                DO NPI=1,NP
                   TSHIFT = TIME(I)-TSTART(NPI)
                   IF(TSHIFT.LE.0.D0) THEN
                      DC = 0.D0
                   ELSE
                      CALL MODEL1(XC(JJ),YC(II),ZC(KK),TSHIFT,C)
                      DC = C*DELC(NPI)
                   END IF
                   SUMC = SUMC+DC
                end do
C
                loc=jj+(ii-1)*nx+(kk-1)*nx*ny
                conc(loc) = SUMC
C
              end do
            end do
          END DO
C
C         write out concentration distribution to ASCII file
C         ==================================================
          WRITE(64,3000) Time(i)
          do jj=1,nx
            do ii=1,ny
              do kk=1,nz
                loc=jj+(ii-1)*nx+(kk-1)*nx*ny
                WRITE(64,3000) xc(jj),yc(ii),zc(kk),conc(loc)
                RCONC(loc) = REAL(CONC(loc))
              end do
            end do
          end do
C
C         write out concentration distribution to unformatted file
C         ========================================================
          T2 = REAL(TIME(I))
          do ilay=nz,1,-1
             LayRev=nz-ilay+1
             WRITE(65) ntrans,kstp,kper,t2,text,nx,ny,LayRev
             write(65) (( RCONC( jj+(ii-1)*nx+(ilay-1)*nx*ny ),
     &        jj=1,nx),ii=ny,1,-1)
          end do
c
         END DO
c
         close(64)
         close(65)
c
      END IF
C
C     FORMAT STATEMENTS
C     =================
1000  FORMAT(A80)
2000  FORMAT(10X,'ATrans3',/,10X,8('*'),/,
     1  10X,'ANALYTICAL SOLUTION FOR 3-D SOLUTE TRANSPORT',/,
     2  10X,'FROM A PATCH SOURCE WITH TIME-VARYING CONCENTRATION',//
     3  10X,'SEMI-INFINITE IN X: X>=0',/,
     4  10X,'INFINITE IN Y',/,
     5  10X,'FINITE IN Z: 0<=Z<=L',///)
2100  FORMAT(10X,A70,//)
2200  FORMAT(10X,'INPUT DATA',/,10X,10('='),//,
     1  10X,'TRANSPORT PARAMETERS',/10X,20('-'),/,
     2  10X,'AVERAGE LINEAR GROUNDWATER VELOCITY = ',1PE12.3,/,
     3  10X,'LONGITUDINAL DISPERSIVITY           = ',1PE12.3,/,
     4  10X,'HORIZONTAL TRANSVERSE DISPERSIVITY  = ',1PE12.3,/,
     5  10X,'VERTICAL TRANSVERSE DISPERSIVITY    = ',1PE12.3,/,
     6  10X,'EFFECTIVE DIFFUSION COEFFICIENT     = ',1PE12.3)
2300  FORMAT(10X,'AQUIFER THICKNESS                   = ',1PE12.3/)
2400  FORMAT(10X,'SOLUTE PROPERTIES',/10X,17('-'),/,
     1  10X,'CONTAMINANT DECAY CONSTANT          = ',1PE12.3,/,
     2  10X,'RETARDATION FACTOR                  = ',1PE12.3)
2500  FORMAT(/10X,'SOLUTION PARAMETERS',/,10X,19('-'),/,
     1  10X,'NUMBER OF GAUSS POINTS              = ',I4,/,
     2  10X,'NUMBER OF TERMS IN SERIES           = ',I4)
2301  FORMAT(/10X,'PATCH DIMENSIONS',/10X,16('-'),/,
     1  10X,'SOURCE WIDTH                        = ',1PE12.3,/,
     2  10X,'BOTTOM OF SOURCE LOCATED AT Z1      = ',1PE12.3,/,
     3  10X,'TOP OF SOURCE LOCATED AT Z2         = ',1PE12.3/)
2302  format(/10X,'INFLOW CONCENTRATION HISTORY',/10X,28('-'),/,
     1  12X,'   TIME  ',4X,'CONCENTRATION',/)
2303  FORMAT(10X,1PE12.3,2X,1PE12.3)
2600  FORMAT(//10X,'CONSTRUCTED INFLOW CONC. HISTOGRAM',/,
     1  10X,'==================================',/,
     2  18X,'TIME INTERVAL',10X,'CONCENTRATION',/)
2700  FORMAT(10X,1PE12.3,' - ',1PE12.3,3X,1PE12.3)
2800  FORMAT(10X,1PE12.3,' --> INFINITY ',4X,1PE12.3)
2900  FORMAT(13X,'TIME',11X,'X',12X,'Y',12X,'Z',11X,' C  '
     2  /13X,'----',10X,'---',10X,'---',10X,'---',10X,'---')
3000  FORMAT(7X,5(1X,1PE12.3))
c
C     END MAIN PROGRAM
C     ================
      STOP
      END
C
C     ****************************
      SUBROUTINE MODEL1(X,Y,Z,T,C)
C     ****************************
C
      INCLUDE 'ATRANS.INC'
c
C     DECLARATION OF VARIABLES
C     ========================
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      COMMON /TDATA/VR,DXR,DYR,DZR,CLAMDA,
     1              Y0,THICK,Z1,Z2,IFLAGPP
      common /sparam/ngaus,nfour
C
      DOUBLE PRECISION ETA(MAXGAUS),W(MAXGAUS)
C
C     SET CONSTANTS
C     -------------
      SMALLX = 1.D-6
      SMALLY = 1.D-6
      PI     = 4.D0*DATAN(1.D0)
C
c     evaluate solution
c     =================
      IF(T.LE.0.d0) THEN
        C = 0.d0              ! initial conditions
      ELSE
C
C       INFLOW BOUNDARY CONDITIONS
C       --------------------------
        IF(X.LE.SMALLX) THEN
          IF((DABS(Y)).LE.Y0.AND.
     1      ((Z.GE.Z1).AND.(Z.LE.Z2))) THEN
            C = 1.D0
          ELSE
            C = 0.D0
          END IF
        ELSE
C
C         EVALUATE THE SOLUTION WITHIN THE DOMAIN
C         USING GAUSS-LEGENDRE QUADRATURE
C         -------------------------------
C
C         A. SCALE THE VARIABLE OF INTEGRATION TO THE 4th POWER
C
          BOT  = 0.D0
          TOP  = DSQRT(DSQRT(T))
C
C         B. ASSEMBLE THE GAUSS QUADRATURE WEIGHTS AND ABSCISSAE
C
          CALL DGAUSS(BOT,TOP,ETA,W,NGAUS)
C
C         C. INITIALIZE THE SUMMATION FOR THE INTEGRAL
C
          SUMI = 0.D0
C
C         D. LOOP OVER NUMBER OF GAUSS POINTS
C
          DO M=1,NGAUS
             TAU  = ETA(M)
             TAU4 = TAU**4
C
C            EVALUATE X TERM
C            ===============
             ARGX = (CLAMDA*TAU4)
     1             +((X-(VR*TAU4))**2)/(4.D0*DXR*TAU4)
             IF(ARGX.GT.50.D0) THEN
                XTERM = 0.D0
             ELSE
                XTERM = DEXP(-ARGX)/(TAU**3)
             END IF
C
C            EVALUATE Y TERM
C            ===============
             IF(DABS(Y).LE.SMALLY) THEN
                arg   = Y0/(2.D0*DSQRT(DYR*TAU4))
                YTERM = 2.D0*(1.D0-DERFC(arg))
             ELSE
                arg1  = (Y-Y0)/(2.D0*DSQRT(DYR*TAU4))
                arg2  = (Y+Y0)/(2.D0*DSQRT(DYR*TAU4))
                YTERM = DERFC(arg1)-derfc(arg2)
             END IF
C
C            EVALUATE Z TERM
C            ===============
             IF(IFLAGPP.EQ.0) THEN
                ZTERM = Z2-Z1
             ELSE
C
C               EVALUATE FOURIER SERIES
C               -----------------------
C               A. CALCULATE CONSTANTS
C
                PIZ1L = (pi/thick)*Z1
                PIZ2L = (pi/thick)*Z2
                PIZL  = (PI/thick)*Z
C
C               B. INITIALIZE SUMMATION
C
                sumn = 0.d0
C
C               C. LOOP OVER NUMBER OF TERMS IN THE SERIES
C
                DO NN=1,NFOUR
                   dN    = DBLE(NN)
                   ARGNN = ((DN*PI/THICK)**2)*DZR*TAU4
                   IF(ARGNN.GT.50.D0) THEN
                      TERMN = 0.D0
                   ELSE
                      ANG1  = DMOD(dN*PIZ1L,2.D0*PI)
                      ANG2  = DMOD(dN*PIZ2L,2.D0*PI)
                      ANG3  = DMOD(dN*PIZL,2.D0*PI)
                      TERMN = DEXP(-ARGNN)*(DSIN(ANG2)-DSIN(ANG1))
     1                        *DCOS(ANG3)/dn
                   END IF
                   sumn = sumn+termn
                end do
                ZTERM = (Z2-Z1)+(2.D0*THICK/PI)*SUMN
C
             END IF
C
C            ASSEMBLE COMPLETE SOLUTION
C            ==========================
             TERM = XTERM*YTERM*ZTERM
C
C            E. EVALUATE THIS TERM OF THE QUADRATURE AND UPDATE THE INTEGRAL
C
             TERMI = TERM*W(M)
             SUMI  = SUMI+TERMI
          end do
C
C         CALCULATE THE CONCENTRATION
C         ===========================
          C = (X/(4.d0*THICK*DSQRT(PI*DXR)))
     1        *(4.D0*SUMI)
        END IF
      END IF
C
      RETURN
      END
c
c     ********************
      include 'derfc.for'
      include 'dgauss.for'
C
C     ****************************
      SUBROUTINE CHK1(Z1,Z2,THICK)
C     ****************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     CHECK PATCH LOCATION
C     --------------------
      IF(Z1.LT.0.0) STOP 'Z1 must be >= 0'
      IF(Z2.gt.thick) STOP 'Z2 must be <= thick'
      IF(Z1.ge.z2) STOP 'Z1 must be < z2'
C
      return
      END
C
C     ****************************
      SUBROUTINE CHK2(NGAUS,nfour)
C     ****************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'ATRANS.INC'
C
C     CHECK SOLUTION PARAMETERS
C     -------------------------
      IF(NGAUS.LE.0) STOP 'No Gauss points requested'
      IF(NGAUS.GT.MAXGAUS) STOP
     1                    'Too many Gauss points requested'
c
      IF(NFOUR.le.0) STOP 'No Fourier series terms requested'
C
      return
      END
C
C     ***********************
      SUBROUTINE CHK3(NP,TSI)
C     ***********************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'ATRANS.INC'
      DIMENSION TSI(NP)
C
C     CHECK TIME HISTORY DATA
C     -----------------------
      IF(NP.GT.MAXNP) THEN
         STOP 'Too many history points times specified'
      ELSEIF(NP.LE.0) THEN
         STOP 'No history points specified'
      ENDIF
C
      DO 40 NPI=1,NP
         IF(TSI(NPI).LT.0.0) STOP 'Invalid time history value'
40    CONTINUE
c
      RETURN
      END
C
C     ************************************************
      SUBROUTINE CHK4(NOBS,XI,ZI,thick,TMIN,TMAX,DELT)
C     ************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'ATRANS.INC'
      DIMENSION XI(NOBS),ZI(NOBS)
C
C     CHECK NUMBER OF OBSERVATION POINTS
C     ==================================
      IF(NOBS.GT.MAXOBS) THEN
         STOP 'Too many observation points requested'
      END IF
c
c     check locations
c     ===============
      DO 10 I=1,Nobs
         IF(xi(I).LT.0.0) STOP 'Invalid x-location'
         IF(zi(I).LT.0.0) STOP 'Invalid z-location'
         IF(zi(I).gt.thick) STOP 'Invalid z-location'
10    CONTINUE
c
c     check times
c     ===========
      IF(tMIN.LT.0.0) STOP 'tMIN must be >= 0'
      IF(tMAX.LT.tMIN) STOP 'tMAX must be >= tMIN'
      IF(DELt.LE.0.0) STOP 'DELt must be > 0'
C
      RETURN
      END
C
C     **********************************************
      SUBROUTINE CHK5(XMIN,XMAX,DELX,YMIN,YMAX,DELY,
     1                ZMIN,ZMAX,DELZ,THICK)
C     ******************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'ATRANS.INC'
C
C     CHECK X-LOCATIONS
C     =================
      IF(XMIN.LT.0.0) STOP 'XMIN must be >= 0'
      IF(XMAX.LT.XMIN) STOP 'XMAX must be >= XMIN'
      IF(DELX.LE.0.0) STOP 'DX must be > 0'
C
C     CHECK Y-LOCATIONS
C     =================
      IF(yMAX.LT.yMIN) STOP 'YMAX must be >= YMIN'
      IF(DELY.LE.0.0) STOP 'DY must be > 0'
C
C     CHECK Z-LOCATIONS
C     =================
      IF(ZMIN.LT.0.0) STOP 'ZMIN must be >= 0'
      IF(ZMAX.LT.ZMIN) STOP 'ZMAX must be >= ZMIN'
      IF(ZMAX.GT.THICK) STOP 'ZMAX must be <= THICK'
      IF(DELZ.LE.0.0) STOP 'DZ must be > 0'
C
      RETURN
      END
C
C     **************************************
      SUBROUTINE CHK6(NTIMES,TIME,NX,NY,NZ)
C     **************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'ATRANS.INC'
      DIMENSION TIME(NTIMES)
C
C     CHECK TIMES
C     ===========
      IF(NTIMES.GT.MAXT) THEN
         STOP 'Too many times requested'
      END IF
C
      DO 10 I=1,NTIMES
         IF(TIME(I).LT.0.0) STOP 'Invalid time value'
10    CONTINUE
C
C     CHECK DIMENSIONS
C     ================
      IF(NX.GT.MAXX) THEN
         STOP 'Too many X points requested'
      ELSEIF(NX.LE.0) THEN
         STOP 'No X points requested'
      ENDIF
C
      IF(NY.GT.MAXY) THEN
         STOP 'Too many Y points requested'
      ELSEIF(NY.LE.0) THEN
         STOP 'No Y points requested'
      ENDIF
C
      IF(NZ.GT.MAXZ) THEN
         STOP 'Too many Z points requested'
      ELSEIF(NZ.LE.0) THEN
         STOP 'No Z points requested'
      ENDIF
C
      RETURN
      END
